@@ -13,20 +13,49 @@ use Scalar::Util qw(reftype);
 use File::Slurp;
 use Clone qw(clone);
 use Data::Dumper;
+use Net::IP::XS;
+use Log::Any qw( $log );
+use YAML::XS qw();
+use Scalar::Util qw( looks_like_number );
+
+$YAML::XS::Boolean = "JSON::PP";
 
-use Zonemaster::Engine::Net::IP;
 use Zonemaster::Engine::Constants qw( $RESOLVER_SOURCE_OS_DEFAULT $DURATION_5_MINUTES_IN_SECONDS $DURATION_1_HOUR_IN_SECONDS $DURATION_4_HOURS_IN_SECONDS $DURATION_12_HOURS_IN_SECONDS $DURATION_1_DAY_IN_SECONDS $DURATION_1_WEEK_IN_SECONDS $DURATION_180_DAYS_IN_SECONDS );
 
 my %profile_properties_details = (
+    q{cache} => {
+        type    => q{HashRef},
+        test    => sub {
+            my @allowed_keys = ( 'redis' );
+            foreach my $cache_database ( keys %{$_[0]} ) {
+                if ( not grep( /^$cache_database$/, @allowed_keys ) ) {
+                    die "Property cache keys have " . scalar @allowed_keys . " possible values: " . join(", ", @allowed_keys);
+                }
+
+                if ( not scalar keys %{ $_[0]->{$cache_database} } ) {
+                    die "Property cache.$cache_database has no items";
+                }
+                else {
+                    my @allowed_subkeys;
+                    if ( $cache_database eq 'redis' ) {
+                        @allowed_subkeys = ( 'server', 'expire' );
+                    }
+
+                    foreach my $key ( keys %{ $_[0]->{$cache_database} } ) {
+                        if ( not grep( /^$key$/, @allowed_subkeys ) ) {
+                            die "Property cache.$cache_database subkeys have " . scalar @allowed_subkeys . " possible values: " . join(", ", @allowed_subkeys);
+                        }
+
+                        die "Property cache.$cache_database.$key has a NULL or empty item" if not $_[0]->{$cache_database}->{$key};
+                        die "Property cache.$cache_database.$key has a negative value" if ( looks_like_number( $_[0]->{$cache_database}->{$key} ) and $_[0]->{$cache_database}->{$key} < 0 ) ;
+                    }
+                }
+            }
+        }
+    },
     q{resolver.defaults.debug} => {
         type    => q{Bool}
     },
-    q{resolver.defaults.dnssec} => {
-        type    => q{Bool}
-    },
-    q{resolver.defaults.edns_size} => {
-        type    => q{Num}
-    },
     q{resolver.defaults.igntc} => {
         type    => q{Bool}
     },
@@ -56,13 +85,28 @@ my %profile_properties_details = (
         type    => q{Str},
         test    => sub {
             if ( $_[0] ne $RESOLVER_SOURCE_OS_DEFAULT ) {
-                eval { Zonemaster::Engine::Net::IP->new( $_[0] ) };
-                if ( $@ ) {
-                    die "Property resolver.source must be an IP address or the exact string $RESOLVER_SOURCE_OS_DEFAULT";
-                }
+                Net::IP::XS->new( $_[0] ) || $log->warning( "Property resolver.source must be an IP address or the exact string $RESOLVER_SOURCE_OS_DEFAULT" );
             }
         }
     },
+    q{resolver.source4} => {
+        type    => q{Str},
+        test    => sub {
+            if ( $_[0] and $_[0] ne '' and not Net::IP::XS::ip_is_ipv4( $_[0] ) ) {
+                $log->warning( "Property resolver.source4 must be an IPv4 address, the empty string or undefined" );
+            }
+            Net::IP::XS->new( $_[0] );
+        }
+    },
+    q{resolver.source6} => {
+        type    => q{Str},
+        test    => sub {
+            if ( $_[0] and $_[0] ne '' and not Net::IP::XS::ip_is_ipv6( $_[0] ) ) {
+                $log->warning( "Property resolver.source6 must be an IPv6 address, the empty string or undefined" );
+            }
+            Net::IP::XS->new( $_[0] );
+        }
+    },
     q{net.ipv4} => {
         type    => q{Bool}
     },
@@ -260,9 +304,18 @@ sub default {
             $new->set( $property_name, $profile_properties_details{$property_name}{default} );
         }
     }
+    $new->check_validity;
     return $new;
 }
 
+sub check_validity {
+    my ( $self ) = @_;
+    my $resolver = $self->{profile}{resolver};
+    if ( exists $resolver->{source} and ( exists $resolver->{source4} or exists $resolver->{source6} ) ) {
+        $log->warning( "Error in profile: 'resolver.source' (deprecated) can't be used in combination with 'resolver.source4' or 'resolver.source6'." );
+    }
+}
+
 sub get {
     my ( $self, $property_name ) = @_;
 
@@ -362,6 +415,7 @@ sub merge {
             $self->_set( q{JSON}, $property_name, _get_value_from_nested_hash( $other_profile->{q{profile}}, split /[.]/, $property_name ) );
         }
     }
+    $self->check_validity;
     return $other_profile->{q{profile}};
 }
 
@@ -377,6 +431,7 @@ sub from_json {
         }
     }
 
+    $new->check_validity;
     return $new;
 }
 
@@ -386,6 +441,18 @@ sub to_json {
     return encode_json( $self->{q{profile}} );
 }
 
+sub from_yaml {
+    my ( $class, $yaml ) = @_;
+    my $data = YAML::XS::Load( $yaml );
+    return $class->from_json( encode_json( $data ) );
+}
+
+sub to_yaml {
+    my ( $self ) = @_;
+
+    return YAML::XS::Dump( $self->{q{profile}} );
+}
+
 sub effective {
     return $effective;
 }
@@ -419,7 +486,7 @@ section.
 Here is an example for updating the effective profile with values from
 a given file and setting all properties not mentioned in the file to
 default values.
-For details on the file format see the L</JSON REPRESENTATION> section.
+For details on the file format see the L</REPRESENTATIONS> section.
 
     use Zonemaster::Engine::Profile;
 
@@ -495,8 +562,29 @@ Dies if the given string is illegal according to the L</JSON REPRESENTATION>
 section or if the property values are illegal according to the L</PROFILE
 PROPERTIES> section.
 
+=head2 from_yaml
+
+A constructor that returns a new profile with values parsed from a YAML string.
+
+    my $profile = Zonemaster::Engine::Profile->from_yaml( <<EOF
+    no_network: true
+    EOF
+    );
+
+The returned profile has set values for all properties specified in the
+given string.
+The remaining properties are unset.
+
+Dies if the given string is illegal according to the L</YAML REPRESENTATION>
+section or if the property values are illegal according to the L</PROFILE
+PROPERTIES> section.
+
 =head1 INSTANCE METHODS
 
+=head2 check_validity
+
+Verify that the profile does not allow confusing combinations.
+
 =head2 get
 
 Get the value of a property.
@@ -543,6 +631,14 @@ Serialize the profile to the L</JSON REPRESENTATION> format.
 
 Returns a string.
 
+=head2 to_yaml
+
+Serialize the profile to the L</JSON REPRESENTATION> format.
+
+    my $string = $profile->to_yaml();
+
+Returns a string.
+
 =head1 SUBROUTINES
 
 =head2 _get_profile_paths
@@ -592,10 +688,6 @@ A boolean. If true, only use TCP. Default false.
 An integer between 1 and 255 inclusive. The number of seconds between retries.
 Default 3.
 
-=head2 resolver.defaults.dnssec
-
-A boolean. If true, sets the DO flag in queries. Default false.
-
 =head2 resolver.defaults.recurse
 
 A boolean. If true, sets the RD flag in queries. Default false.
@@ -625,11 +717,27 @@ replay, set this flag to false.
 
 =head2 resolver.source
 
+Deprecated (planned removal: v2024.1).
+Use L</resolver.source4> and L</resolver.source6>.
 A string that is either an IP address or the exact string C<"os_default">.
 The source address all resolver objects should use when sending queries.
 If C<"os_default">, the OS default address is used.
 Default C<"os_default">.
 
+=head2 resolver.source4
+
+A string that is an IPv4 address or the empty string or undefined.
+The source address all resolver objects should use when sending queries over IPv4.
+If the empty string or undefined, use the OS default IPv4 address if available.
+Default "" (empty string).
+
+=head2 resolver.source6
+
+A string that is an IPv6 address or the empty string or undefined.
+The source address all resolver objects should use when sending queries over IPv6.
+If the empty string or undefined, use the OS default IPv6 address if available.
+Default "" (empty string).
+
 =head2 net.ipv4
 
 A boolean. If true, resolver objects are allowed to send queries over
@@ -649,12 +757,10 @@ cache.
 
 =head2 asnroots (DEPRECATED)
 
-An arrayref of domain names. Default C<["asnlookup.zonemaster.net",
-"asnlookup.iis.se", "asn.cymru.com"]>.
+An arrayref of domain names. Default C<["asnlookup.zonemaster.net"]>.
 
 The domains will be assumed to be Cymru-style AS lookup zones.
-Normally only the first name in the list will be used, the rest are
-backups in case the earlier ones don't work.
+Only the first name in the list will be used.
 
 =head2 asn_db.style
 
@@ -665,11 +771,28 @@ Default C<"Cymru">.
 =head2 asn_db.sources
 
 An arrayref of domain names when asn_db.style is set to C<"Cymru"> or whois
-servers when asn_db.style is set to C<"RIPE">. Normally only the first item
-in the list will be used, the rest are backups in case the earlier ones don't
-work.
+servers when asn_db.style is set to C<"RIPE">. Only the first item
+in the list will be used.
 Default C<"asnlookup.zonemaster.net">.
 
+=head2 cache (EXPERIMENTAL)
+
+A hash of hashes. The currently supported keys are C<"redis">.
+
+See more information in L<cache.redis>.
+
+Undefined by default.
+
+=head2 cache.redis (EXPERIMENTAL)
+
+A hashref. The currently supported keys are C<"server"> and C<"expire">.
+
+Specifies the address of the Redis server used to perform global caching
+(C<cache.redis.server>) and an optional expire time (C<cache.redis.expire>).
+
+C<cache.redis.server> must be a string in the form C<host:port>.
+C<cache.redis.expire> must be a non-negative integer and defines a time in seconds. Default 5 seconds.
+
 =head2 logfilter
 
 A complex data structure. Default C<{}>.
@@ -778,20 +901,17 @@ level for the tag.
 
 =head2 test_cases
 
-An arrayref of names of implemented test cases as listed in the
-L<test case specifications|
-https://github.com/zonemaster/zonemaster/tree/master/docs/specifications/tests/ImplementedTestCases.md>.
+An arrayref of names of implemented test cases (in all lower-case) as listed in the
+L<test case specifications|https://github.com/zonemaster/zonemaster/tree/master/docs/specifications/tests/ImplementedTestCases.md>.
 Default is an arrayref listing all the test cases.
 
-Specifies which test cases to consider when a test module is asked
-to run of all of its test cases.
+Specifies which test cases can be run by the testing suite.
 
-Test cases not included here can still be run individually.
-
-The test cases C<basic00>, C<basic01> and C<basic02> are always considered no
-matter if they're excluded from this property.
-This is because part of their function is to verify that the given domain name
-can be tested at all.
+Note that an exception applies to test cases C<basic01> and C<basic02>:
+when running either the full testing suite or just the Basic test module,
+these test cases are always run no matter if they're excluded from this
+property. This is because their primary goal is to verify that the given
+domain name can be tested at all.
 
 =head2 test_cases_vars.dnssec04.REMAINING_SHORT
 
@@ -829,7 +949,7 @@ https://github.com/zonemaster/zonemaster/blob/master/docs/specifications/tests/Z
 Related to the REFRESH_MINIMUM_VALUE_LOWER message tag from this test case.
 Default C<14400> (4 hours in seconds).
 
-=head2 test_cases_vars04.zone.SOA_RETRY_MINIMUM_VALUE
+=head2 test_cases_vars.zone04.SOA_RETRY_MINIMUM_VALUE
 
 A positive integer value.
 Recommended lower bound for SOA retry values (in seconds) in test case
@@ -865,7 +985,9 @@ https://github.com/zonemaster/zonemaster/blob/master/docs/specifications/tests/Z
 Related to the SOA_DEFAULT_TTL_MAXIMUM_VALUE_HIGHER message tag from this test case.
 Default C<86400> (1 day in seconds).
 
-=head1 JSON REPRESENTATION
+=head1 REPRESENTATIONS
+
+=head2 JSON REPRESENTATION
 
 Property names in L</PROFILE PROPERTIES> section correspond to paths in
 a datastructure of nested JSON objects.
@@ -888,8 +1010,8 @@ C<net.ipv6> = true has this JSON representation:
         }
     }
 
-=over
+=head2 YAML REPRESENTATION
 
-=back
+Similar to the L</JSON REPRESENTATION> but uses a YAML format.
 
 =cut
